unit rkResTrans;

{$mode objfpc}{$H+}

{
Make sure to include at least LCLStrConsts.id.po and your project.id.po in
Project Options -> Resources (where 'id' is the resp. language id).

If there are other units to translate include the *.id.po _and_ provide a
list of these additional unit names in project resources as well as a simple
text file with a line for each unit, eg TransList.txt. A default value of
'ResTrans' (cf csDefaultList) will be checked anyway. So if you provide all
necessary po-Files and a list of additional units which transfers to a
resource named 'RESTRANS' the standard translation will translate to the
default language of the target machine without any additional parameters.

Always let the IDE take care of the naming of resources so that for example
project.id.po becomes 'PROJECT.ID', LCLStrConsts.id.po -> 'LCLSTRCONSTS.ID'
and TransList.txt -> 'TRANSLIST'.

In case of a custom list name or custom language id call TranslateFromResources
like this:

TranslateFromResources(id,'TransList',True); // force update of existing forms

If you only need standard translation just include rkDefTrans in your uses
(like unit DefaultTranslator supplied with Lazarus).

These routines won't throw an error if there are no resources, but of course
they will also do nothing at all...
}

interface

uses Classes, SysUtils;

procedure TranslateFromResources(ForceUpdate: Boolean=True);
procedure TranslateFromResources(ALangID: string; ForceUpdate: Boolean=True);
procedure TranslateFromResources(ALangID, AUnitList: string; ForceUpdate: Boolean=True);

procedure TranslateUnitFromResource(const AUnitName: string; ALangID: string='');

implementation

uses Forms, LCLType, LResources, LCLTranslator, Translations, LazUTF8;

const
  csLCLStrConsts = 'LCLStrConsts';
  csDefaultList = 'ResTrans';

procedure TranslateFromResources(ForceUpdate: Boolean);
begin
  TranslateFromResources('','',ForceUpdate);
end;

procedure TranslateFromResources(ALangID: string; ForceUpdate: Boolean);
begin
  TranslateFromResources(ALangID,'',ForceUpdate);
end;

procedure TranslateFromResources(ALangID, AUnitList: string; ForceUpdate: Boolean);
var LocalTranslator: TUpdateTranslator;
    Stream: TResourceStream;
    StrList: TStringList;
    ResName: string;
    POFile: TPOFile;
    hNd: THandle;
    i: integer;
begin

  LocalTranslator := nil;

  // ShortLanguageID is sufficient in most cases, if you need more or want
  // to process start parameters provide a more sophisticated method here
  if (ALangID = '') then
    LazGetShortLanguageID(ALangID);

  //first look for application po-file
  ResName := UpperCase(ChangeFileExt(ExtractFileName(ParamStrUTF8(0)),'.'+ALangID));
  hNd := FindResource(hInstance,PChar(ResName),RT_RCDATA);
  if (hNd <> 0) then
  begin
    Stream := TResourceStream.Create(hInstance,ResName,RT_RCDATA);
    try
      POFile := TPOFile.Create(Stream);
      TranslateResourceStrings(POFile);
      // the TPOTranslator takes care of this POFile - so no freeing it
      LocalTranslator := TPOTranslator.Create(POFile);
    finally
      Stream.Free;
    end;
  end;

  // now translate LCLStrConsts
  TranslateUnitFromResource(csLCLStrConsts,ALangID);

  // if no list is supplied try default list
  if (AUnitList = '') then
    AUnitList := csDefaultList;

  // finally work through the list if present
  ResName := UpperCase(AUnitList);
  hNd := FindResource(hInstance,PChar(ResName),RT_RCDATA);
  if (hNd <> 0) then
  begin
    Stream := TResourceStream.Create(hInstance,ResName,RT_RCDATA);
    try
      StrList := TStringList.Create;
      try
        StrList.LoadFromStream(Stream);
        for i:=0 to pred(StrList.Count) do
          if (StrList.Strings[i] > '') then
            TranslateUnitFromResource(StrList.Strings[i],ALangID);
      finally
        StrList.Free;
      end;
    finally
      Stream.Free;
    end;
  end;

  // assign the new standard translator
  if Assigned(LocalTranslator) then
  begin
    if Assigned(LRSTranslator) then
      LRSTranslator.Free;
    LRSTranslator := LocalTranslator;
  end;

  // as stated in SetDefaultLang() this might not
  // be safe for calls from initialization section or .lpr!
  // (tested on Linux - it is... and two more checks:)
  if ForceUpdate and Assigned(Screen) and Assigned(LocalTranslator) then
  begin
    for i:=0 to pred(Screen.CustomFormCount) do
      LocalTranslator.UpdateTranslation(Screen.CustomForms[i]);
    for i:=0 to pred(Screen.DataModuleCount) do
      LocalTranslator.UpdateTranslation(Screen.DataModules[i]);
  end;
end;

procedure TranslateUnitFromResource(const AUnitName: string; ALangID: string='');
var Stream: TResourceStream;
    ResName: string;
    POFile: TPOFile;
    hNd: THandle;
begin
  if (ALangID = '') then
    LazGetShortLanguageID(ALangID);
  ResName := UpperCase(AUnitName+'.'+ALangID);
  hNd := FindResource(hInstance,PChar(ResName),RT_RCDATA);
  if (hNd <> 0) then
  begin
    Stream := TResourceStream.Create(hInstance,ResName,RT_RCDATA);
    try
      POFile := TPOFile.Create(Stream);
      try
        TranslateUnitResourceStrings(AUnitName,POFile);
      finally
        POFile.Free;
      end;
    finally
      Stream.Free;
    end;
  end;
end;

end.

